"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-var-requires */
exports.default = {
    command: 'start',
    describe: 'start a server, worker or both (default both)',
    aliases: ['s'],
    builder: {
        worker: {
            default: false,
            boolean: true,
            describe: 'background job processing only',
        },
        stats: {
            default: false,
            boolean: true,
            describe: 'print stats and exit',
        },
        server: {
            default: false,
            boolean: true,
            describe: 'serve traffic but no background job processing (can enqueue jobs)',
        },
    },
    handler(argv) {
        return __awaiter(this, void 0, void 0, function* () {
            const { runApp, useWorkers } = require('../../');
            const { worker, server, root, stats } = argv;
            if (worker) {
                return useWorkers({
                    root,
                }, ({ context }) => __awaiter(this, void 0, void 0, function* () {
                    var _a;
                    const store = context.store();
                    const logger = store.logger;
                    logger.info('running in worker mode');
                    const bg = (_a = store.workers) === null || _a === void 0 ? void 0 : _a.background;
                    if (bg === 'inprocess') {
                        logger.warn('you are running workers inprocess (workers.inprocess=true), worker runner does not make sense.');
                    }
                    if (stats) {
                        yield context.helpers.printJobCounts(context.workers());
                        process.exit(0);
                    }
                }));
            }
            else {
                return runApp({
                    root,
                    workersMode: server ? 'producer' : 'duplex',
                });
            }
        });
    },
};
