"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieConfig = exports.loadConfig = exports.resolveConfig = void 0;
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const debug_1 = __importDefault(require("debug"));
const context_1 = require("./context");
const utils_1 = require("./utils");
const debug = (0, debug_1.default)('hyperstack:config');
const resolveConfig = ({ exactPath, appPath }) => {
    if (exactPath) {
        return exactPath;
    }
    if (!appPath) {
        throw new Error('app path is empty');
    }
    return path_1.default.join(appPath, `config/environments/${(0, context_1.getAppEnvironment)()}`);
};
exports.resolveConfig = resolveConfig;
const loadConfig = (paths) => __awaiter(void 0, void 0, void 0, function* () {
    const resolved = (0, exports.resolveConfig)(paths);
    const mod = (0, utils_1.requireModule)(resolved);
    // configs are function providing a dict.
    // it's a function because merging modifies the provided dict, and we
    // don't want to fiddle with module cache in case we want live reloads or tests to work
    const config = (yield mod());
    debug('config before overrides is %o', config);
    if (paths.configOverrides) {
        lodash_1.default.merge(config, paths.configOverrides);
    }
    return { config, resolvedConfigPath: resolved };
});
exports.loadConfig = loadConfig;
const tieConfig = (context, { exactPath, configOverrides } = {}) => __awaiter(void 0, void 0, void 0, function* () {
    const appPath = context.store().app.root;
    const { config, resolvedConfigPath } = yield (0, exports.loadConfig)({
        configOverrides,
        exactPath,
        appPath,
    });
    debug('loaded from %o', resolvedConfigPath);
    debug('config is %o', config);
    debug('overrides %o', configOverrides);
    context.setConfig(config);
});
exports.tieConfig = tieConfig;
