"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieApp = exports.resolveAppRoot = exports.getAppEnvironment = exports.getEnvironment = exports.envMode = exports.isDevelopment = exports.isProduction = exports.isTest = exports.context = void 0;
const path_1 = __importDefault(require("path"));
const debug_1 = __importDefault(require("debug"));
const lodash_1 = __importDefault(require("lodash"));
const keys_1 = require("./keys");
const utils_1 = require("./utils");
const debug = (0, debug_1.default)('hyperstack:context');
class InMemContext {
    constructor() {
        this.__store = {};
        this.__config = null;
        this.clear = () => {
            this.__store = {};
            this.__config = null;
        };
        this.baseUrl = () => this.__store.controllers.baseUrl;
        this.logger = () => this.__store.logger;
        this.models = () => this.__store.models.models;
        this.mailers = () => this.__store.workers.mailers;
        this.workers = () => this.__store.workers.workers;
        this.controllers = () => this.__store.controllers.controllers;
        this.environment = () => this.__store.app.environmentFile;
        this.helpers = {
            printRoutes: utils_1.printRoutesV2,
            printJobCounts: utils_1.printJobCounts,
        };
        this.update = (updater, d) => {
            debug('[%o] context update: %o', updater, d);
            lodash_1.default.merge(this.__store, d);
        };
        this.setConfig = (c) => {
            this.__config = c;
        };
        this.store = () => this.__store;
        this.config = () => {
            if (!this.__config) {
                throw new Error('configuration not loaded yet.');
            }
            return this.__config;
        };
        this.initializerProps = (key) => { var _a; return lodash_1.default.get(((_a = this.__store.initializers) === null || _a === void 0 ? void 0 : _a.props) || {}, key); };
    }
}
const context = new InMemContext();
exports.context = context;
const isTest = () => process.env.NODE_ENV === 'test';
exports.isTest = isTest;
const isProduction = () => process.env.NODE_ENV === 'production';
exports.isProduction = isProduction;
const isDevelopment = () => !(0, exports.isTest)() && !(0, exports.isProduction)();
exports.isDevelopment = isDevelopment;
const envMode = () => (0, exports.isTest)() ? 'test' : (0, exports.isProduction)() ? 'production' : 'development';
exports.envMode = envMode;
const getEnvironment = () => lodash_1.default.replace(process.env.NODE_ENV || 'development', /[^A-Za-z0-9_-]/g, '');
exports.getEnvironment = getEnvironment;
const getAppEnvironment = () => process.env[keys_1.consts.appEnvKey]
    ? lodash_1.default.replace(process.env[keys_1.consts.appEnvKey], /[^A-Za-z0-9_-]/g, '')
    : (0, exports.getEnvironment)();
exports.getAppEnvironment = getAppEnvironment;
const resolveAppRoot = (app) => app || path_1.default.join(process.cwd(), '/src');
exports.resolveAppRoot = resolveAppRoot;
const tieApp = (context, app) => {
    const appRoot = (0, exports.resolveAppRoot)(app);
    context.update(keys_1.setters.context, {
        app: {
            mode: (0, exports.envMode)(),
            environmentFile: (0, exports.getAppEnvironment)(),
            root: appRoot,
        },
    });
};
exports.tieApp = tieApp;
