"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieControllers = exports.getControllerConfig = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const hypercontroller_1 = require("@hyperstackjs/hypercontroller");
const keys_1 = require("./keys");
const utils_1 = require("./utils");
const VAN_HALEN_PORT = 5150;
const resolvePort = () => process.env.PORT || VAN_HALEN_PORT;
const createApp = ({ controllers, initializers, logging, staticFolder, indexCatchAll, forceHttps, gzip, cookieSecret, sendValidationErrors, urlencoded, json }) => {
    return (0, hypercontroller_1.createServer)({
        controllers,
        opts: {
            logging,
            staticFolder,
            indexCatchAll,
            forceHttps,
            gzip,
            cookieSecret,
            sendValidationErrors,
            urlencoded,
            json,
            beforeMiddleware: (preapp) => {
                if (initializers && initializers.beforeMiddleware) {
                    initializers.beforeMiddleware.forEach((init) => init(preapp));
                }
            },
            afterMiddleware: (postapp) => {
                if (initializers && initializers.afterMiddleware) {
                    initializers.afterMiddleware.forEach((init) => init(postapp));
                }
            },
            beforeControllers: (preapp) => {
                if (initializers && initializers.beforeControllers) {
                    initializers.beforeControllers.forEach((init) => init(preapp));
                }
            },
            afterControllers: (postapp) => {
                if (initializers && initializers.afterControllers) {
                    initializers.afterControllers.forEach((init) => init(postapp));
                }
            },
        },
    });
};
const getControllerConfig = (context) => {
    var _a;
    const store = context.store();
    const appRoot = store.app.root;
    const logger = store.logger;
    const middleware = store.logging.middleware;
    const initializers = (_a = store.initializers) === null || _a === void 0 ? void 0 : _a.initializers;
    const controllerPath = path_1.default.join(appRoot, 'app', 'controllers');
    const { serveStatic, indexCatchAll, forceHttps, gzip, cookieSecret, sendValidationErrors, json, helmet, urlencoded, baseUrl: cBaseUrl, } = context.config().controllers || {};
    const baseUrl = cBaseUrl || `http://localhost:${resolvePort()}`;
    if (!fs_1.default.existsSync(controllerPath)) {
        throw new Error(`tie controllers: no app controller in ${controllerPath}`);
    }
    const { AppController } = (0, utils_1.requireModule)(controllerPath);
    if (!AppController) {
        throw new Error(`No 'AppController' export was found in ${controllerPath}`);
    }
    const staticFolder = serveStatic ? path_1.default.join(appRoot, 'public') : undefined;
    return {
        controllers: AppController.controllers,
        initializers,
        logging: { logger, middleware },
        staticFolder,
        indexCatchAll,
        forceHttps,
        gzip,
        cookieSecret,
        sendValidationErrors,
        helmet,
        json,
        urlencoded,
        baseUrl,
    };
};
exports.getControllerConfig = getControllerConfig;
const tieControllers = (context) => __awaiter(void 0, void 0, void 0, function* () {
    const appConfig = (0, exports.getControllerConfig)(context);
    const app = createApp(appConfig);
    context.update(keys_1.setters.controllers, {
        controllers: {
            numControllers: appConfig.controllers.length,
            baseUrl: appConfig.baseUrl,
            controllers: appConfig.controllers,
            port: resolvePort(),
            app,
            didTieControllers: true,
        },
    });
});
exports.tieControllers = tieControllers;
