"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieInitializers = exports.getInitializers = exports.initializer = void 0;
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const debug_1 = __importDefault(require("debug"));
const utils_1 = require("./utils");
const keys_1 = require("./keys");
const debug = (0, debug_1.default)('hyperstack:initializers');
const initializer = (fn) => {
    return fn;
};
exports.initializer = initializer;
const getInitializers = (appRoot) => {
    const initpath = path_1.default.join(appRoot, 'config/initializers/*.[tj]s');
    return (0, utils_1.load)(initpath) || [];
};
exports.getInitializers = getInitializers;
const tieInitializers = (context) => __awaiter(void 0, void 0, void 0, function* () {
    const appRoot = context.store().app.root;
    const initializerCreators = (0, exports.getInitializers)(appRoot);
    const initializers = [];
    for (const initializerCreator of initializerCreators) {
        initializers.push(yield initializerCreator(context));
    }
    // into initializermap, array initializers per lifecycle hook:
    // [{beforeApp:}, {beforeApp:}] -> { beforeApp: [..], ... afterApp: [..]}
    const inits = lodash_1.default.mergeWith({}, ...initializers, (v1, v2) => lodash_1.default.isUndefined(v1) ? [v2] : [...v1, v2]);
    inits.beforeApp && inits.beforeApp.forEach((init) => init());
    const props = {};
    inits.provideProps &&
        inits.provideProps.forEach((provide) => {
            const p = provide();
            debug('providing props: %o', p);
            lodash_1.default.merge(props, p);
        });
    context.update(keys_1.setters.initializers, {
        initializers: { initializers: inits, props },
    });
});
exports.tieInitializers = tieInitializers;
