"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieModels = exports.runSeed = exports.seed = void 0;
const path_1 = __importDefault(require("path"));
const hypermodel_1 = require("@hyperstackjs/hypermodel");
const keys_1 = require("./keys");
const utils_1 = require("./utils");
const seed = (fn) => fn;
exports.seed = seed;
const runSeed = (app) => __awaiter(void 0, void 0, void 0, function* () {
    const { context } = app;
    const appPath = context.store().app.root;
    const seedfile = path_1.default.join(appPath, 'config', 'db', 'seed');
    const seedfn = (0, utils_1.requireModule)(seedfile);
    yield seedfn(app);
});
exports.runSeed = runSeed;
const tieModels = (context, migrationsMode) => __awaiter(void 0, void 0, void 0, function* () {
    const store = context.store();
    const logger = store.logger;
    const appRoot = store.app.root;
    const modelsRoot = path_1.default.join(appRoot, 'app', 'models/index.[tj]s');
    const config = context.config().database;
    const { models, connection } = yield (0, hypermodel_1.buildModels)({
        logger,
        config,
        modelsRoot,
    });
    if (!models) {
        logger.info(`No models loaded (no models exported in ${modelsRoot} or no such file)`);
    }
    // this brings the DB up to date, using sync, truncate, migrations, etc.
    // based on configuration and state of migrations
    let didTieMigrations = false;
    if (migrationsMode !== 'none') {
        const migrationsRoot = path_1.default.join(appRoot, 'config', 'db', 'migrate', '*.[tj]s');
        const resolvedConfig = migrationsMode === 'run' ? Object.assign(Object.assign({}, config), { migrate: true }) : config;
        try {
            yield (0, hypermodel_1.converge)({
                migrationsRoot,
                config: resolvedConfig,
                models,
                connection,
                logger,
            });
        }
        catch (e) {
            logger.error(e);
            throw e;
        }
        didTieMigrations = true;
    }
    context.update(keys_1.setters.models, {
        models: {
            models,
            connection,
            didTieModels: true,
            didTieMigrations,
        },
    });
});
exports.tieModels = tieModels;
