"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieTasks = exports.getTaskmap = exports.getTasks = exports.createTasks = exports.taskfileToName = exports.loadTasks = exports.printTasks = exports.taskmapTable = exports.task = void 0;
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const colorette_1 = require("colorette");
const utils_1 = require("./utils");
const keys_1 = require("./keys");
const task = (desc, fn) => {
    return {
        description: desc,
        exec: fn,
    };
};
exports.task = task;
const taskmapTable = (tasksRoot, taskmap) => {
    return `\n${(0, colorette_1.bold)((0, colorette_1.white)('Tasks'))}\n\n${lodash_1.default.map(taskmap, (v, k) => `${(0, colorette_1.underline)((0, colorette_1.yellow)(k))}: ${v.description}`).join('\n')}`;
};
exports.taskmapTable = taskmapTable;
const printTasks = (tasksRoot, taskmap, printfn = console.log // eslint-disable-line no-console
) => {
    printfn((0, exports.taskmapTable)(tasksRoot, taskmap));
};
exports.printTasks = printTasks;
const loadTasks = (tasksPath) => {
    const tasks = path_1.default.join(tasksPath, '**/*.[tj]s');
    return (0, utils_1.loadMap)(tasks) || {};
};
exports.loadTasks = loadTasks;
const taskfileToName = (tasksRoot, k) => path_1.default
    .normalize(k)
    .replace(tasksRoot, '')
    .replace(/^[\\/]/, '')
    .replace(/\.[tj]s$/, '');
exports.taskfileToName = taskfileToName;
const createTasks = (tasksRoot) => {
    const m = (0, exports.loadTasks)(tasksRoot);
    const taskmap = lodash_1.default.mapKeys(m, (_v, k) => (0, exports.taskfileToName)(tasksRoot, k));
    return taskmap;
};
exports.createTasks = createTasks;
const getTasks = (context) => {
    const store = context.store();
    return {
        root: store.tasks.root,
        map: store.tasks.map,
    };
};
exports.getTasks = getTasks;
const getTaskmap = (context) => {
    return (0, exports.getTasks)(context).map;
};
exports.getTaskmap = getTaskmap;
const tieTasks = (context) => {
    var _a;
    const appPath = context.store().app.root;
    const customTasksRoot = (_a = context.config().tasks) === null || _a === void 0 ? void 0 : _a.root;
    const tasksRoot = customTasksRoot
        ? path_1.default.join(appPath, customTasksRoot)
        : path_1.default.join(appPath, 'lib', 'tasks');
    const taskMap = (0, exports.createTasks)(tasksRoot);
    context.update(keys_1.setters.tasks, {
        tasks: {
            root: tasksRoot,
            map: taskMap,
        },
    });
};
exports.tieTasks = tieTasks;
