"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runApp = exports.useSeed = exports.useTasks = exports.useControllers = exports.useModels = exports.useApp = exports.usePortal = exports.useWorkers = exports.createUseApp = exports.shutDown = exports.createApp = void 0;
const hypercontroller_1 = require("@hyperstackjs/hypercontroller");
const debug_1 = __importDefault(require("debug"));
const config_1 = require("./config");
const controllers_1 = require("./controllers");
const context_1 = require("./context");
const logger_1 = require("./logger");
const workers_1 = require("./workers");
const models_1 = require("./models");
const initializers_1 = require("./initializers");
const utils_1 = require("./utils");
const tasks_1 = require("./tasks");
const debug = (0, debug_1.default)('hyperstack:tie');
const createApp = ({ root, configFile, configOverrides, workersMode = 'duplex', migrationsMode = 'auto', controllersMode = 'auto', }) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    const started = new Date().getTime();
    context_1.context.clear();
    // root
    (0, context_1.tieApp)(context_1.context, root);
    // config
    yield (0, config_1.tieConfig)(context_1.context, {
        exactPath: configFile,
        configOverrides,
    });
    // initializers
    yield (0, initializers_1.tieInitializers)(context_1.context);
    // logger
    const logger = (0, logger_1.tieLogger)(context_1.context);
    // models, migration and database schema sync
    yield (0, models_1.tieModels)(context_1.context, migrationsMode);
    // workers
    if (workersMode !== 'none') {
        yield (0, workers_1.tieWorkers)(context_1.context, workersMode);
    }
    if (controllersMode !== 'none' && workersMode !== 'consumer') {
        // controllers
        yield (0, controllers_1.tieControllers)(context_1.context);
    }
    // context.validate()
    yield (0, tasks_1.tieTasks)(context_1.context);
    const app = (_b = context_1.context.store().controllers) === null || _b === void 0 ? void 0 : _b.app;
    context_1.context.update('performance', {
        stats: {
            bootTime: new Date().getTime() - started,
        },
    });
    // server can be null if we're in consumer mode
    return { app, context: context_1.context, logger };
});
exports.createApp = createApp;
const shutDown = (context) => __awaiter(void 0, void 0, void 0, function* () {
    var _c, _d, _e;
    const store = context.store();
    const backend = (_c = store.workers) === null || _c === void 0 ? void 0 : _c.backend;
    if (backend && backend.stop) {
        try {
            debug('shutting down backend connection');
            yield backend.stop();
        }
        catch (e) {
            debug('error while stopping workers backend: %o', e);
        }
    }
    const workersConnection = (_d = store.workers) === null || _d === void 0 ? void 0 : _d.connection;
    if (workersConnection && workersConnection.quit) {
        try {
            debug('shutting down workers connection');
            yield workersConnection.quit();
        }
        catch (e) {
            debug('error while closing workers connection: %o', e);
        }
    }
    const modelsConnection = (_e = store.models) === null || _e === void 0 ? void 0 : _e.connection;
    if (modelsConnection && modelsConnection.close()) {
        try {
            debug('shutting down models connection');
            yield modelsConnection.close();
        }
        catch (e) {
            debug('error while closing models connection: %o', e);
        }
    }
});
exports.shutDown = shutDown;
const createUseApp = (defaultOpts) => (appOpts, fn = (_a) => { }) => __awaiter(void 0, void 0, void 0, function* () {
    const resolvedOpts = Object.assign(Object.assign({}, defaultOpts), appOpts);
    debug('create app, opts: %o', resolvedOpts);
    const res = yield (0, exports.createApp)(resolvedOpts);
    try {
        yield fn(res);
        if (resolvedOpts.isTask) {
            yield (0, exports.shutDown)(res.context);
        }
        return res;
    }
    catch (e) {
        if ((0, context_1.isTest)()) {
            yield (0, exports.shutDown)(res.context);
            throw e;
        }
        console.log('run error', e); // eslint-disable-line no-console
        process.exit(1);
    }
});
exports.createUseApp = createUseApp;
// don't migrate, and let controllers load depending on what the workers mode
// are (producer, consumer, or duplex)
exports.useWorkers = (0, exports.createUseApp)({
    migrationsMode: 'none',
    workersMode: 'consumer',
    controllersMode: 'auto',
    isTask: false,
});
exports.usePortal = (0, exports.createUseApp)({
    migrationsMode: 'none',
    workersMode: 'producer',
    controllersMode: 'auto',
    isTask: false,
});
// don't do anything other than loading models.
exports.useApp = (0, exports.createUseApp)({
    migrationsMode: 'none',
    workersMode: 'none',
    controllersMode: 'none',
    isTask: true,
});
exports.useModels = (0, exports.createUseApp)({
    migrationsMode: 'none',
    workersMode: 'producer',
    controllersMode: 'none',
    isTask: true,
});
exports.useControllers = (0, exports.createUseApp)({
    migrationsMode: 'none',
    workersMode: 'none',
    controllersMode: 'auto',
    isTask: true,
});
const useTasks = (appOpts, yargsParsed) => __awaiter(void 0, void 0, void 0, function* () {
    return (0, exports.useModels)(appOpts, (app) => __awaiter(void 0, void 0, void 0, function* () {
        const { map: taskmap, root: taskRoot } = (0, tasks_1.getTasks)(app.context);
        const args = yargsParsed;
        const showTasks = !!args.list;
        const [taskname] = args._;
        debug('tie task args: %o', args);
        debug('tie taskname: %o', taskname);
        debug('tie taskmap: %o', taskmap);
        const task = taskname && taskmap[taskname];
        // show tasks
        if (showTasks || !task) {
            (0, tasks_1.printTasks)(taskRoot, taskmap, console.log); // eslint-disable-line no-console
        }
        else {
            debug('tie task: %o', taskname);
            const res = yield task.exec(args, app);
            debug('tie task result: %o', res);
            return res;
        }
    }));
});
exports.useTasks = useTasks;
const useSeed = (appOpts) => __awaiter(void 0, void 0, void 0, function* () { return (0, exports.useModels)(appOpts, models_1.runSeed); });
exports.useSeed = useSeed;
const runApp = (appOpts) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { app, context } = yield (0, exports.createApp)(appOpts);
        const port = context.store().controllers.port;
        (0, utils_1.printBanner)(context, console.log); // eslint-disable-line no-console
        (0, hypercontroller_1.listen)(app, { port, log: () => { } });
    }
    catch (e) {
        console.log('run error', e); // eslint-disable-line no-console
        process.exit(1);
    }
});
exports.runApp = runApp;
