"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.printBanner = exports.printRoutesV2 = exports.printJobCounts = exports.exportRoutes = exports.loadMap = exports.load = exports.requireModule = exports.sleep = void 0;
/* eslint-disable no-console */
const fs_1 = __importDefault(require("fs"));
const glob_1 = require("glob");
const colorette_1 = require("colorette");
const lodash_1 = __importDefault(require("lodash"));
const hypercontroller_1 = require("@hyperstackjs/hypercontroller");
const as_table_1 = __importDefault(require("as-table"));
const hyperworker_1 = require("@hyperstackjs/hyperworker");
const sleep = (t) => new Promise((resolve, _reject) => setTimeout(() => resolve(true), t));
exports.sleep = sleep;
const requireModule = (f) => {
    let mod = null;
    try {
        mod = require(f);
    }
    catch (e) {
        console.log(`error dynamically loading '${f}'\n${e.toString()}`);
        throw e;
    }
    return mod.default || mod;
};
exports.requireModule = requireModule;
const load = (patt) => lodash_1.default.sortBy((0, glob_1.globSync)(patt), lodash_1.default.identity).map(exports.requireModule);
exports.load = load;
const loadMap = (patt) => lodash_1.default.sortBy((0, glob_1.globSync)(patt), lodash_1.default.identity).reduce((acc, f) => {
    acc[f] = (0, exports.requireModule)(f);
    return acc;
}, {});
exports.loadMap = loadMap;
const exportRoutes = (controllers, toFile) => {
    const routemap = (0, hypercontroller_1.getRouteMap)(controllers);
    const spec = (0, hypercontroller_1.routemapToOpenAPI)(routemap);
    fs_1.default.writeFileSync(toFile, JSON.stringify(spec.getSpec(), null, 2));
};
exports.exportRoutes = exportRoutes;
const snipmap = {
    wget: 'shell_wget',
    curl: 'shell_curl',
    httpie: 'shell_httpie',
};
const normalizeSnippets = (snip) => snipmap[snip] || snip;
const printJobCounts = (workers) => __awaiter(void 0, void 0, void 0, function* () {
    const counts = yield (0, hyperworker_1.getJobCounts)(Object.values(workers));
    console.log(as_table_1.default.configure({
        print: (v, k) => {
            if (k === 'name') {
                return (0, colorette_1.bold)((0, colorette_1.yellow)(v)).toString();
            }
            else if (k === 'failed') {
                return (0, colorette_1.red)(v).toString();
            }
            else if (k === 'active') {
                return (0, colorette_1.yellow)(v).toString();
            }
            else if (k === 'completed') {
                return (0, colorette_1.green)(v).toString();
            }
            else {
                return (0, colorette_1.gray)(v).toString();
            }
        },
    })(counts));
});
exports.printJobCounts = printJobCounts;
const printRoutesV2 = (controllers, snippets = []) => {
    const routemap = (0, hypercontroller_1.getRouteMap)(controllers);
    // print the route map
    lodash_1.default.forEach(routemap, (mapping, ctl) => {
        console.log(`${(0, colorette_1.green)(ctl)} ${(0, colorette_1.gray)(mapping.path)}`);
        mapping.actions.forEach((action) => {
            action.routes.forEach((route) => {
                const verb = route.verb;
                const path = (0, hypercontroller_1.asOpenAPIPath)(route.parsedPath);
                console.log(`  ${(0, colorette_1.yellow)(verb.toUpperCase())}\t${(0, colorette_1.whiteBright)(path)}`);
                if (snippets && snippets.length > 0) {
                    const snips = (0, hypercontroller_1.getRouteSnippets)(routemap, verb, path, snippets.map(normalizeSnippets));
                    snips.snippets.forEach((snip) => {
                        console.log(``);
                        console.log(`${(0, colorette_1.underline)((0, colorette_1.bold)((0, colorette_1.white)(snip.title)))}`);
                        console.log(`${(0, colorette_1.gray)(snip.content)}`);
                        console.log(``);
                    });
                }
            });
        });
        console.log('\n');
    });
};
exports.printRoutesV2 = printRoutesV2;
const printBanner = (context, print) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    const logger = context.logger();
    const { bold, green, cyan, underline } = logger.colors;
    const stat = (s) => underline(bold(green(s)));
    const setting = (s) => bold(cyan(s));
    const store = context.store();
    const port = (_a = store.controllers) === null || _a === void 0 ? void 0 : _a.port;
    const displayOn = (cfg) => Object.keys(lodash_1.default.omitBy(cfg, (v) => !v))
        .map((v) => setting(v))
        .join(', ');
    const banner = `

  ${(0, colorette_1.cyanBright)(`▄                             ▄             ▄ ▄`)}
  ${cyan(`█─▄  ▄ ▄  ▄─▄  ▄─▄  ▄─▄  ▄──  █─   ─▄  ▄──  █─▄`)}
  ${(0, colorette_1.magentaBright)(`▀ ▀  ▀─█  █─▀  ▀──  ▀    ──▀  ▀─  ▀─▀  ▀──  ▀ ▀`)}
  ${(0, colorette_1.magentaBright)(`     └─▀${(0, colorette_1.gray)(`              https://hyperstackjs.io`)}`)}


🚀 App is running on port ${stat(port)}, in ${stat((_b = store.app) === null || _b === void 0 ? void 0 : _b.mode)} mode. 
🤘 Loaded ${stat((_c = store.controllers) === null || _c === void 0 ? void 0 : _c.numControllers)} controllers, ${stat((_d = store.workers) === null || _d === void 0 ? void 0 : _d.numWorkers)} workers, ${stat((_e = store.workers) === null || _e === void 0 ? void 0 : _e.numMailers)} mailers in ${(0, colorette_1.magenta)(parseInt((_f = store.stats) === null || _f === void 0 ? void 0 : _f.bootTime))}ms.

🧰 Environment is environments/${stat((_g = store.app) === null || _g === void 0 ? void 0 : _g.environmentFile)}
   · logger: ${setting(logger.level)}
   · controllers: ${displayOn(context.config().controllers)}
   · workers: ${setting((_h = store.workers) === null || _h === void 0 ? void 0 : _h.background)}, ${setting((_j = store.workers) === null || _j === void 0 ? void 0 : _j.mode)}
   · models:  ${displayOn(context.config().database)}
   · mailers: ${displayOn(lodash_1.default.omit(context.config().mailers, 'delivery'))} delivery=${setting(((_k = context.config().mailers) === null || _k === void 0 ? void 0 : _k.delivery) || 'none')}
`;
    print(banner);
};
exports.printBanner = printBanner;
