"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tieWorkers = exports.getWorkersConfig = void 0;
const path_1 = __importDefault(require("path"));
const hyperworker_1 = require("@hyperstackjs/hyperworker");
const keys_1 = require("./keys");
const getWorkersConfig = (context) => {
    const store = context.store();
    const config = context.config();
    const logger = store.logger;
    const appRoot = store.app.root;
    if (!config.workers) {
        throw new Error('cannot configure workers and mailers. app config is not initialized');
    }
    const mailerSettings = config.mailers;
    const { inprocess: isInProcess, backendOpts, redis: redisUrl, truncate, } = config.workers;
    const mailersGlob = path_1.default.join(appRoot, 'app', 'mailers', 'index.[tj]s');
    const mailerTemplatesRoot = path_1.default.join(appRoot, 'app', 'mailers');
    const workersGlob = path_1.default.join(appRoot, 'app', 'workers', 'index.[tj]s');
    const initializers = store.initializers.initializers;
    const beforeWorkers = (workerTypes) => {
        initializers &&
            initializers.beforeWorkers &&
            initializers.beforeWorkers.forEach((init) => init(workerTypes));
    };
    return {
        isInProcess,
        workersGlob,
        mailersGlob,
        mailerTemplatesRoot,
        mailerSettings,
        logger,
        redisUrl,
        truncate,
        backendOpts,
        beforeWorkers,
        afterWorkers: (_p) => { }, // not used
    };
};
exports.getWorkersConfig = getWorkersConfig;
const tieWorkers = (context, mode) => __awaiter(void 0, void 0, void 0, function* () {
    const ctx = yield (0, hyperworker_1.buildWorkers)((0, exports.getWorkersConfig)(context), mode);
    context.update(keys_1.setters.workers, {
        workers: Object.assign(Object.assign({}, ctx), { didTieWorkers: true }),
    });
});
exports.tieWorkers = tieWorkers;
